
def adjust_constants_in_source_code(queue_logging, settings):
    """
    :param queue_logging: queue_logging: queue for logging
    :param settings: object containing all important settings
    :return: return 0 by success else raise an error
    this function manipulates the hook source code by changing the port to the port given in NaviX_settings
    """
    try:
        queue_logging.put("DEBUG: constants in hook source code file: " + str(settings.source_path_hook_main))
        hook_file = ""
        with open(settings.source_path_hook_main, "r") as fin:
            for line in fin:
                if line.find("int portNum =") != -1:
                    line = "int portNum = " + str(settings.port) + ";\n"
                elif line.find("string code_HPDA_RANK") != -1:
                    line = "string code_HPDA_RANK = " + "\"" + str(settings.hpda_rank) + "\";\n"
                elif line.find("string code_HPDA_REQUIREMENT") != -1:
                    line = "string code_HPDA_REQUIREMENT = " + "\"" + str(settings.hpda_requirement) + "\";\n"
                elif line.find("string code_Rank") != -1:
                    line = "string code_Rank = " + "\"" + str(settings.rank) + "\";\n"
                elif line.find("string code_directory") != -1:
                    line = "string code_directory = " + "\"" + str(settings.working_directory) + "\";\n"
                hook_file += line
        fin.close()
        with open(settings.source_path_hook_main, "w") as out:
            for line in hook_file:
                out.write(line)
        out.close()
        queue_logging.put("DEBUG: " + "Finished editing source code of main.h.")
        hook_file = ""
        with open(settings.source_path_update_hook, "r") as fin:
            for line in fin:
                if line.find("string code_HPDA_TIMESTAMP =") != -1:
                    line = "string code_HPDA_TIMESTAMP = " + "\"" + str(settings.hpda_timestamp) + "\";\n"
                hook_file += line
        fin.close()
        with open(settings.source_path_update_hook, "w") as out:
            for line in hook_file:
                out.write(line)
        out.close()
        queue_logging.put("DEBUG: " + "Finished editing source code of update hook.")
        hook_file = ""
        with open(settings.source_path_finalize_hook, "r") as fin:
            for line in fin:
                if line.find("string code_HPDA_TIMESTAMP =") != -1:
                    line = "string code_HPDA_TIMESTAMP = " + "\"" + str(settings.hpda_timestamp) + "\";\n"
                elif line.find("string code_start_job_timestamp =") != -1:
                    line = "string code_start_job_timestamp = " + "\"" + str(settings.start_job_timestamp) + "\";\n"
                elif line.find("string code_completion_timestamp =") != -1:
                    line = "string code_completion_timestamp = " + "\"" + str(settings.completion_timestamp) + "\";\n"
                elif line.find("string code_File_found_on_server =") != -1:
                    line = "string code_File_found_on_server = " + "\"" + str(settings.file_found_on_server) + "\";\n"
                elif line.find("string code_Requested_files =") != -1:
                    line = "string code_Requested_files = " + "\"" + str(settings.requested_files_mon) + "\";\n"
                elif line.find("string code_HPDA_RANK_MONITORING =") != -1:
                    line = "string code_HPDA_RANK_MONITORING = " + "\"" + str(settings.hpda_rank_monitoring) + "\";\n"
                elif line.find("string code_GlobalJobId =") != -1:
                    line = "string code_GlobalJobId = " + "\"" + str(settings.global_job_id) + "\";\n"
                elif line.find("string code_RemoteUserCpu =") != -1:
                    line = "string code_RemoteUserCpu = " + "\"" + str(settings.remote_user_cpu) + "\";\n"
                elif line.find("string code_RemoteSysCpu =") != -1:
                    line = "string code_RemoteSysCpu = " + "\"" + str(settings.remote_sys_cpu) + "\";\n"
                elif line.find("string code_RequestCpus =") != -1:
                    line = "string code_RequestCpus = " + "\"" + str(settings.requested_cpus) + "\";\n"
                elif line.find("string code_NetworkInputMb =") != -1:
                    line = "string code_NetworkInputMb = " + "\"" + str(settings.network_input_mb) + "\";\n"
                elif line.find("string code_LastRemoteHost =") != -1:
                    line = "string code_LastRemoteHost = " + "\"" + str(settings.last_remote_host) + "\";\n"
                hook_file += line
        fin.close()
        queue_logging.put("DEBUG: " + "Finished editing source code of finalize hook.")
        queue_logging.put("DEBUG: source codes successfully read and changed constants in file: "
                          + str(settings.source_path_hook_main) + ", " + str(settings.source_path_update_hook)
                          + " and " + str(settings.source_path_finalize_hook))
        with open(settings.source_path_finalize_hook, "w") as out:
            for line in hook_file:
                out.write(line)
        out.close()
        queue_logging.put("DEBUG: source code successfully saved under: " + str(settings.source_path_hook_main))
        return 0
    except Exception as e:
        queue_logging.put("ERROR: Error while changing constants source files! " +
                          "-> stopping server!\n" +
                          "\nCheck path to the hook given in 'NaviX.cnf'" +
                          ",\n the rights of file access" +
                          " and the code itself, given path was: " + str(settings.source_path_hook_main)
                          + "\nPython-ERROR: " + str(e))
        raise ValueError("Failed to adjust source code")


def actual_compile_hook(source, target, queue_logging):
    """
    :param source: path with name of source file
    :param target: path with name of executable
    :param queue_logging: queue for logging
    :return: return 0 by success else raise an error
    compiles the hook (written in C++), source and output file are configured in NaviX_settings
    """
    try:
        import subprocess
        queue_logging.put("DEBUG: Start compiling")
        bash_command = "g++ -Wall " + str(source) + " -o " + str(target)
        process = subprocess.Popen(bash_command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output = process.stderr.read()
        process.stdout.close()
        process.stderr.close()
        if output != b'':
            queue_logging.put("ERROR: " + str(output))
            raise ValueError("Failed to compile hook!")
        queue_logging.put("DEBUG: Compiled successfully to: " + str(target))
        return 0
    except Exception as e:
        queue_logging.put("ERROR: Error while compiling the hook! Source: " + str(source) + " Target: " + str(target)
                          + "\nPython-ERROR: " + str(e))
        raise ValueError("Failed to compile hook!")


def set_hook_as_executable(target, queue_logging):
    """
    :param target: path and name to executable which should be modified to be executable
    :param queue_logging: queue for logging
    :return: return 0 by success else raise an error
    sets the compiled hook as executable
    """
    try:
        queue_logging.put("DEBUG: set hook as executable (chmod +x " + str(target) + ")")
        import subprocess
        bash_command = "chmod +x " + str(target)
        process = subprocess.Popen(bash_command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output = process.stderr.read()
        process.stdout.close()
        process.stderr.close()
        if output != b'':
            queue_logging.put("ERROR: " + str(output))
            raise ValueError("Failed to set hook as executable!")
        queue_logging.put("DEBUG: Successfully set hook as executable (" + str(target) + ")")
        return 0
    except Exception as e:
        queue_logging.put("ERROR: Error while setting hook as executable! ("
                          + str(target) + ")\nPython-ERROR: " + str(e))
        raise ValueError("Failed to set hook as executable!")


def start_compile_hook(queue_logging, settings):
    """
    :param queue_logging: queue for logging
    :param settings: object containing all important settings
    :return: return 0 by success else raise an error
    to prevent issues with access limit per file and speed limit the port of the NaviX service is hard coded into the
    hooks, but to provide variable port for NaviX this function manipulates the hook source code by changing the port
    to the port given in NaviX_settings. Afterwards the hook is compiled and set as executable.
    """
    queue_logging.put("DEBUG: Start compiling")
    try:
        adjust_constants_in_source_code(queue_logging, settings)
    except Exception as err:
        raise ValueError("Failed to adjust constants in source code!\n" + str(err))
    try:
        actual_compile_hook(settings.source_path_translate_hook,
                            settings.target_compile_translate_hook, queue_logging)
    except Exception as err:
        raise ValueError("Failed to compile translate hook!\n" + str(err))
    try:
        actual_compile_hook(settings.source_path_update_hook,
                            settings.target_compile_update_hook, queue_logging)
    except Exception as err:
        raise ValueError("Failed to compile update hook!\n" + str(err))
    try:
        actual_compile_hook(settings.source_path_finalize_hook,
                            settings.target_compile_finalize_hook, queue_logging)
    except Exception as err:
        raise ValueError("Failed to compile finalize hook!\n" + str(err))
    try:
        set_hook_as_executable(settings.target_compile_translate_hook, queue_logging)
    except Exception as err:
        raise ValueError("Failed to set execution flag to translate hook!\n" + str(err))
    try:
        set_hook_as_executable(settings.target_compile_update_hook, queue_logging)
    except Exception as err:
        raise ValueError("Failed to set execution flag to update hook!\n" + str(err))
    try:
        set_hook_as_executable(settings.target_compile_finalize_hook, queue_logging)
    except Exception as err:
        raise ValueError("Failed to set execution flag to finalize hook!\n" + str(err))
    queue_logging.put("DEBUG: Successfully compiled")
    return 0
