#!/usr/bin/env python3.5
from time import time
from navix.request_pyxrootd import create_pyxrootd_client, stat_for_files_pyxrootd
from navix.common import find_string_between


class Monitor:

    def __init__(self, queue_monitor, queue_logging, thread_num, settings):
        self.settings = settings
        self.thread_num = thread_num
        self.queue_logging = queue_logging
        self.queue_monitor = queue_monitor
        self.requested_files = ""
        self.files_server_dict = dict()
        self.rank_server = dict()
        self.requirement_server = ""
        self.timestamp = ""
        self.remote_host = ""
        self.job_id = ""
        self.CompletionDate = 0
        self.JobCurrentStartDate = 0
        self.RequestCpus = 0
        self.RemoteSysCpu = 0
        self.RemoteUserCpu = 0
        self.NetworkInputMb = 0
        self.Input_Files = []
        self.remote_ip = ""

    def finalize(self, data):
        next_line_input_files = False
        self.queue_logging.put("DEBUG: " + "Received data: " + data)
        for line in data.splitlines():
            if self.settings.hpda_timestamp in line[:len(self.settings.hpda_timestamp)] and not next_line_input_files:
                self.timestamp = find_string_between(self.queue_logging, self.thread_num, line, "\"", "\"")
            elif self.settings.last_remote_host in line[:len(self.settings.last_remote_host)] \
                    and not next_line_input_files:
                self.remote_host = find_string_between(self.queue_logging, self.thread_num, line, "\"", "\"")
            elif self.settings.last_remote_ip in line[:len(self.settings.last_remote_ip)] \
                    and not next_line_input_files:
                self.remote_ip = find_string_between(self.queue_logging, self.thread_num, line, "/", "\"")
            elif self.settings.network_input_mb in line[:len(self.settings.network_input_mb)] \
                    and not next_line_input_files:
                self.NetworkInputMb = float(line.split("= ")[1])
            elif self.settings.global_job_id in line[:len(self.settings.global_job_id)] and not next_line_input_files:
                self.job_id = find_string_between(self.queue_logging, self.thread_num, line, "#", "#")
            elif self.settings.start_job_timestamp in line[:len(self.settings.start_job_timestamp)]\
                    and not next_line_input_files:
                try:
                    self.JobCurrentStartDate = float(line.split("= ")[1])
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get float from JobCurrentStartDate!\n"
                                           + "Python-ERROR: " + str(e))
            elif self.settings.completion_timestamp in line[:len(self.settings.completion_timestamp)] \
                    and not next_line_input_files:
                try:
                    self.CompletionDate = float(line.split("= ")[1])
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get float from CompletionDate!\n"
                                           + "Python-ERROR: " + str(e))
            elif self.settings.file_found_on_server in line[:len(self.settings.file_found_on_server)] \
                    and not next_line_input_files:
                try:
                    self.files_server_dict = eval(find_string_between(self.queue_logging, self.thread_num,
                                                                      line.split("= ")[1], "\"", "\""))
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get dict of files found on server!\n"
                                           + "Python-ERROR: " + str(e))

            elif self.settings.remote_user_cpu in line[:len(self.settings.remote_user_cpu)] \
                    and not next_line_input_files:
                try:
                    self.RemoteUserCpu = float(line.split("= ")[1])
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get RemoteUserCpu!\n"
                                           + "Python-ERROR: " + str(e))

            elif self.settings.remote_sys_cpu in line[:len(self.settings.remote_sys_cpu)] and not next_line_input_files:
                try:
                    self.RemoteSysCpu = float(line.split("= ")[1])
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get RemoteSysCpu!\n"
                                           + "Python-ERROR: " + str(e))

            elif self.settings.requested_cpus in line[:len(self.settings.requested_cpus)] and not next_line_input_files:
                try:
                    self.RequestCpus = float(line.split("= ")[1])
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get RequestCpus!\n"
                                           + "Python-ERROR: " + str(e))
            elif self.settings.requested_files_tmp in line[:len(self.settings.requested_files_tmp)] \
                    or next_line_input_files:
                try:
                    if self.settings.requested_files_tmp + " = " in line[:len(self.settings.requested_files_tmp + " = ")]:
                        line = line.replace(self.settings.requested_files_tmp + " = \"", "")
                    if 'END_OF_REQUESTED_FILES' in line[-len('END_OF_REQUESTED_FILES'):]:
                        next_line_input_files = False
                    else:
                        next_line_input_files = True
                        self.Input_Files.extend(line.split(","))
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get list of input files (string with server name)!\n"
                                           + "Python-ERROR: " + str(e))

            elif self.settings.requested_files_mon in line[:len(self.settings.requested_files_mon)]:
                try:
                    self.requested_files = find_string_between(self.queue_logging,
                                                               self.thread_num, line.split("= ")[1], "\"", "\"")
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get list of requested files!\n"
                                           + "Python-ERROR: " + str(e))
            elif self.settings.hpda_rank_monitoring in line[:len(self.settings.hpda_rank_monitoring)] \
                    and not next_line_input_files:
                try:
                    self.rank_server = eval((line.split(self.settings.hpda_rank_monitoring + " = \"")[1])[:-1])
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to evaluate rank dictionary!\n"
                                           + "Python-ERROR: " + str(e))
            elif self.settings.hpda_requirement in line[:len(self.settings.hpda_requirement)] \
                    and not next_line_input_files:
                try:
                    self.requirement_server = str(line.split(" = ")[1])
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get requirement string!\n"
                                           + "Python-ERROR: " + str(e))
        if self.Input_Files:
            if 'Input_Files = "' == str(self.Input_Files[0][len('Input_Files = "'):]):
                self.Input_Files[0] = self.Input_Files[0][len('Input_Files = "'):]
            pass
            self.get_file_size()
        self.queue_monitor.put(str(self.timestamp) + ": \n"
                               + "LastRemoteHost: " + self.remote_host + "\n"
                               + "LastRemoteIP: " + self.remote_ip + "\n"
                               + "Done at: " + str(time()) + "\n"
                               + "JobID: " + str(self.job_id) + "\n"
                               + "Requested files: " + str(self.requested_files) + "\n"
                               + "File found on server: " + str(self.files_server_dict) + "\n"
                               + "Rank: " + str(self.rank_server) + "\n"
                               + "Requirement: " + str(self.requirement_server) + "\n"
                               + "Runtime: " + str(self.CompletionDate - self.JobCurrentStartDate) + "\n"
                               + "RequestCpus: " + str(self.RequestCpus) + "\n"
                               + "RemoteSysCpu: " + str(self.RemoteSysCpu) + "\n"
                               + "RemoteUserCpu: " + str(self.RemoteUserCpu) + "\n"
                               + "NetworkInputMb: " + str(self.NetworkInputMb) + "\n"
                               + "Input_Files: " + str(self.Input_Files) + "\n"
                               + "\n")
        self.queue_logging.put("DEBUG: " + str(self.thread_num) + ": "
                               + str(self.timestamp) + ": \n"
                               + "LastRemoteHost: " + self.remote_host + "\n"
                               + "LastRemoteIP: " + self.remote_ip + "\n"
                               + "Done at: " + str(time()) + "\n"
                               + "JobID: " + str(self.job_id) + "\n"
                               + "Requested files: " + str(self.requested_files) + "\n"
                               + "File found on server: " + str(self.files_server_dict) + "\n"
                               + "Rank: " + str(self.rank_server) + "\n"
                               + "Requirement: " + str(self.requirement_server) + "\n"
                               + "Runtime: " + str(self.CompletionDate - self.JobCurrentStartDate) + "\n"
                               + "RequestCpus: " + str(self.RequestCpus) + "\n"
                               + "RemoteSysCpu: " + str(self.RemoteSysCpu) + "\n"
                               + "RemoteUserCpu: " + str(self.RemoteUserCpu) + "\n"
                               + "NetworkInputMb: " + str(self.NetworkInputMb) + "\n"
                               + "Input_Files: " + str(self.Input_Files) + "\n"
                               + "\n")

    def get_file_size(self):
        self.queue_logging.put("DEBUG: " + str(self.thread_num)
                               + ": received requested files in monitoring:\n"
                               + "###########################################\n"
                               + str(self.Input_Files) + ": \n"
                               + "###########################################")
        for index, file_selected in enumerate(self.Input_Files):
            file_selected = file_selected.replace("\"","").replace("\'","")
            try:
                self.queue_logging.put("DEBUG: " + str(self.thread_num) + ": "
                                       + "Requesting statistic data for file: " + str(file_selected)
                                       + "\non server: "
                                       + "root://" + str(find_string_between(self.queue_logging,
                                                                             self.thread_num, file_selected,
                                                                             "root://", "//"))
                                       + "\non default port: " + str(1094) + "(hard coded)")
                my_client = create_pyxrootd_client(self.queue_logging, self.thread_num,
                                                   "root://" + str(find_string_between(self.queue_logging,
                                                                                       self.thread_num, file_selected,
                                                                                       "root://", "//"))
                                                   + ':1094')
                try:
                    status, stat = stat_for_files_pyxrootd(my_client, "/" + file_selected.split("//")[2])
                    del my_client
                    if stat:
                        self.queue_logging.put("DEBUG: " + str(self.thread_num) + ": \n"
                                               + "size of file is: " + str(stat.size) + "\n"
                                               + "request file stat was a: " + str(status) + "\n"
                                               + str(file_selected.split("//")[2]))
                        self.Input_Files[index] = file_selected + ":" + str(stat.size)
                    else:
                        self.queue_logging.put("WARNI: " + str(self.thread_num) + ": \n"
                                               + "request file stat was a: " + str(status) + "\n"
                                               + "requested file is: " + str(file_selected.split("//")[2]))
                        self.Input_Files[index] = file_selected + ":" + "None"
                except Exception as e:
                    self.queue_logging.put("ERROR: " + str(self.thread_num)
                                           + ": Failed to get xrootd statistics for file. Requested file: "
                                           + str(file_selected)
                                           + "statistics for monitoring only\n"
                                           + "Python-ERROR: " + str(e))
            except Exception as e:
                self.queue_logging.put("ERROR: " + str(self.thread_num)
                                       + ": Failed to get xrootd host. This host is needed to request some"
                                       + "statistics for monitoring only\n"
                                       + "Python-ERROR: " + str(e))
