from navix.common import find_string_between

##################################################################################################################
#
# Timestamp operations on startup
#
##################################################################################################################


def check_timestamp(queue_logging, thread_num, data):
    """
    :param queue_logging:
    :param thread_num:
    :param data:
    :return:
    checking for special hpda timestamp ('HPDA_TIMESTAMP') in input data from the hook
    -> if there is a timestamp it is a update_hook
    -> else it is a translate hook -> initial timestamp is created
    """
    try:
        # check given input data for ClassAd 'HPDA_TIMESTAMP' -> if one is found -> update hook
        #                                                     -> else -> translate hook
        last_time = check_data_for_hpda_timestamp(queue_logging, thread_num, data)
        if last_time:
            queue_logging.put("DEBUG: " + str(thread_num)
                              + ": Got timestamp from hook -> update hook, timestamp is: "
                              + str(last_time))
        else:
            queue_logging.put("DEBUG: " + str(thread_num)
                              + ": Got NO timestamp from hook -> translate hook")
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": Error while checking given data if there is already HPDA_TIMESTAMP in classAd!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError
    try:
        # if there is a timestamp in data -> remove time stamp from string data for further computing
        # (only requested files should be left in data now)
        if last_time:
            data = remove_timestamp_from_data(data, last_time)
    except Exception as e:
        queue_logging.put("ERROR: " + str(thread_num)
                          + ": Error while removing HPDA_TIMESTAMP for further work from data!"
                          + "\nPython-ERROR: " + str(e))
        raise ValueError
    return data, last_time


def check_data_for_hpda_timestamp(queue_logging, thread_num, data):
    """
    :param data:
    :param queue_logging:
    :param thread_num:
    :return:
    checks input data from the hook for a timestamp and returns it
    """
    for line in data.splitlines():
        if "HPDA_TIMESTAMP" in line:
            return find_string_between(queue_logging, thread_num, line, "\"", "\"")
    return False


def remove_timestamp_from_data(data, last_time):
    """
    :param data:
    :param last_time:
    :return:
    removes timestamp from input data, so there is no difference afterwards between translate and update hook
    """
    return data.replace("HPDA_TIMESTAMP = \"" + str(last_time) + "\"", "")
