from unittest import TestCase
from subprocess import Popen, PIPE
from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings
from multiprocessing import Queue


##########################################################
# Unittest Block
##########################################################
class TestNaviX(TestCase):
    maxDiff = None

    def setUp(self):
        log_file_s = 'NaviX_start_error.log'
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.INFO)
        self.settings = Settings()

        self.output = Queue()


    def test_stress_test(self):
        process_list = []
        for test_nr in range(1000):
            process_list.append(Process(target=self.translate_hook, args=(self.output,)))
            process_list.append(Process(target=self.update_hook, args=(self.output,)))
            process_list.append(Process(target=self.finalize_hook, args=(self.output,)))
        for process in process_list:
            process.daemon = True
            process.start()
        for process in process_list:
            process.join(0.5)
        for process in process_list:
            self.assertEqual("No Error", str(self.output.get()))

    def translate_hook(self, output):
        try:
            bash_command = self.settings.target_compile_translate_hook
            process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
            input_data = b'one\ntwo\nthree\nfour\nfive\nsix\n\n\n\n'
            output_std, output_error = process.communicate(input=input_data)
            process.kill()
            self.assertEqual(b'', output_error)
            self.assertEqual(b'Header not found / no valid input\n', output_std)

            file_testing = open('tests/test_hooks_files/input_text_testing_hooks.txt', 'r')
            input_data = file_testing.read().encode('utf-8')
            file_testing.close()
            file_testing = open('tests/test_hooks_files/expected_translate.txt', 'r')
            expected_data = file_testing.read().encode('utf-8')
            file_testing.close()
            process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
            output_std, output_error = process.communicate(input=input_data)
            process.kill()
            self.assertEqual(b'', output_error)
            self.assertEqual(expected_data, output_std)
            output.put("No Error")
        except Exception as err:
            output.put(str(err))
        finally:
            exit(0)

    def update_hook(self, output):
        try:
            bash_command = self.settings.target_compile_update_hook
            process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
            input_data = b'one\ntwo\nthree\nfour\nfive\nsix\n\n\n\n'
            output_std, output_error = process.communicate(input=input_data)
            process.kill()
            self.assertEqual(b'', output_error)
            self.assertEqual(b'Requested files not found!\n', output_std)

            file_testing = open('tests/test_hooks_files/input_text_testing_hooks.txt', 'r')
            input_data = file_testing.read().encode('utf-8')
            file_testing.close()
            file_testing = open('tests/test_hooks_files/expected_update.txt', 'r')
            expected_data = file_testing.read().encode('utf-8')
            file_testing.close()
            process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
            output_std, output_error = process.communicate(input=input_data)
            process.kill()
            self.assertEqual(b'', output_error)
            self.assertEqual(expected_data, output_std),
            output.put("No Error")
        except Exception as err:
            output.put(str(err))
        finally:
            exit(0)

    def finalize_hook(self, output):
        try:
            bash_command = self.settings.target_compile_finalize_hook
            process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
            input_data = b'one\ntwo\nthree\nfour\nfive\nsix\n\n\n\n'
            output_std, output_error = process.communicate(input=input_data)
            process.kill()
            self.assertEqual(b'', output_error)
            self.assertEqual(b'', output_std)

            file_testing = open('tests/test_hooks_files/input_text_testing_hooks.txt', 'r')
            input_data = file_testing.read().encode('utf-8')
            file_testing.close()

            process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
            output_std, output_error = process.communicate(input=input_data)
            self.assertEqual(b'', output_error)
            self.assertEqual(b'', output_std)
            output.put("No Error")
        except Exception as err:
            output.put(str(err))
        finally:
            exit(0)
