from unittest import TestCase
from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings

from navix.common import remove_blanks_from_string, find_string_between, is_ip_in_network_range


class TestCommon(TestCase):
    maxDiff = None

    def setUp(self):
        # log file path
        log_file_s = 'NaviX_start_error.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.DEBUG)
        settings = Settings()
        log = Logging()

        self.thread_num = "TEST"
        self.settings = settings
        self.logger = log.queue_logging
        return

    def test_find_string_between(self):
        self.assertEqual(find_string_between(self.logger, self.thread_num, "TestString", "es", "ri"), "tSt")
        self.assertEqual(find_string_between(self.logger, self.thread_num, "TestString", "1", "ri"), "")
        self.assertEqual(find_string_between(self.logger, self.thread_num, "TestString", "es", "1"), "")
        self.assertEqual(find_string_between(self.logger, self.thread_num, "TestString", "1", "1"), "")
        self.logger.put("DEBUG: Tested function: 'test_find_string_between' appears to be fine.")
        return

    def test_remove_blanks_from_string(self):
        self.assertEqual(remove_blanks_from_string(" Tes t Strin g "), "TestString")
        self.assertEqual(remove_blanks_from_string(""), "")
        self.assertEqual(remove_blanks_from_string("  "), "")
        self.assertEqual(remove_blanks_from_string("TestString"), "TestString")
        self.logger.put("DEBUG: Tested function: 'test_remove_blanks_from_string' appears to be fine.")
        return

    def test_is_ip_in_network_range(self):
        self.assertFalse(is_ip_in_network_range('0.0.0.0', '192.178.168.0/28'))
        with self.assertRaises(ValueError):
            is_ip_in_network_range('0.0.0.0', '192.178.168.1/28')
        with self.assertRaises(ValueError):
            is_ip_in_network_range('test', 'nonsense')
        self.assertTrue(is_ip_in_network_range('192.178.168.1', '192.178.168.0/28'))
        self.assertTrue(is_ip_in_network_range('2001:db8::0004', '2001:db8::/123'))
        self.assertFalse(is_ip_in_network_range('2001:db8::0004', '2001:db8::/128'))
