from unittest import TestCase
from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings
from os import access, X_OK

from navix.compile import adjust_constants_in_source_code, actual_compile_hook, set_hook_as_executable,\
    start_compile_hook


class TestCompile(TestCase):
    maxDiff = None

    def setUp(self):
        # log file path
        log_file_s = 'NaviX_start_error.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.INFO)
        log = Logging()

        self.thread_num = "TEST"
        self.settings = Settings()

        self.logger = log.queue_logging
        return

    def test_adjust_constants_in_source_code(self):
        # Test manipulating finalize hook source code
        self.settings.remote_sys_cpu = "blah"
        adjust_constants_in_source_code(self.logger, self.settings)
        adjustment_success = False
        with open(self.settings.source_path_finalize_hook, "r") as fin:
            for line in fin:
                if line.find("string code_RemoteSysCpu = \"blah\";") != -1:
                    adjustment_success = True
        fin.close()
        self.assertTrue(adjustment_success)
        adjustment_success = False
        self.settings.remote_sys_cpu = "RemoteSysCpu"
        adjust_constants_in_source_code(self.logger, self.settings)
        with open(self.settings.source_path_finalize_hook, "r") as fin:
            for line in fin:
                if line.find("string code_RemoteSysCpu = \"RemoteSysCpu\";") != -1:
                    adjustment_success = True
        fin.close()
        self.assertTrue(adjustment_success)

        # Test manipulating main source code
        self.settings.port = "test"
        adjust_constants_in_source_code(self.logger, self.settings)
        adjustment_success = False
        with open(self.settings.source_path_hook_main, "r") as fin:
            for line in fin:
                if line.find("int portNum = test;") != -1:
                    adjustment_success = True
        fin.close()
        self.assertTrue(adjustment_success)
        adjustment_success = False
        self.settings.port = "8889"
        adjust_constants_in_source_code(self.logger, self.settings)
        with open(self.settings.source_path_hook_main, "r") as fin:
            for line in fin:
                if line.find("int portNum = 8889;") != -1:
                    adjustment_success = True
        fin.close()
        self.assertTrue(adjustment_success)

        # Test manipulating update hook source code
        self.settings.hpda_timestamp = "test"
        adjust_constants_in_source_code(self.logger, self.settings)
        adjustment_success = False
        with open(self.settings.source_path_update_hook, "r") as fin:
            for line in fin:
                if line.find("string code_HPDA_TIMESTAMP = \"test\";") != -1:
                    adjustment_success = True
        fin.close()
        self.assertTrue(adjustment_success)
        adjustment_success = False
        self.settings.hpda_timestamp = "HPDA_TIMESTAMP"
        adjust_constants_in_source_code(self.logger, self.settings)
        with open(self.settings.source_path_update_hook, "r") as fin:
            for line in fin:
                if line.find("string code_HPDA_TIMESTAMP = \"HPDA_TIMESTAMP\";") != -1:
                    adjustment_success = True
        fin.close()
        self.assertTrue(adjustment_success)

    def test_actual_compile_hook(self):
        actual_compile_hook(self.settings.source_path_translate_hook, "hooks/tra", self.logger)
        actual_compile_hook(self.settings.source_path_update_hook, "hooks/tra", self.logger)
        actual_compile_hook(self.settings.source_path_finalize_hook, "hooks/tra", self.logger)

    def test_set_hook_as_executable(self):
        set_hook_as_executable("hooks/tra", self.logger)
        self.assertTrue(access("hooks/tra", X_OK))

    def test_start_compile_hook(self):
        self.settings.target_compile_finalize_hook = "hooks/tra"
        self.settings.target_compile_update_hook = "hooks/tra"
        self.settings.target_compile_translate_hook = "hooks/tra"
        start_compile_hook(self.logger, self.settings)
        self.assertTrue(access("hooks/tra", X_OK))