from unittest import TestCase
from subprocess import PIPE, call, Popen
from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings
from time import sleep
from sys import executable


class TestHooks(TestCase):
    maxDiff = None

    def setUp(self):
        self.settings = Settings()

    def test_translate_hook(self):
        bash_command = self.settings.target_compile_translate_hook
        process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
        input_data = b'one\ntwo\nthree\nfour\nfive\nsix\n\n\n\n'
        output_std, output_error = process.communicate(input=input_data)
        process.kill()
        self.assertEqual(b'', output_error)
        self.assertEqual(b'Header not found / no valid input\n', output_std)

        file_testing = open('tests/test_hooks_files/input_text_testing_hooks.txt', 'r')
        input_data = file_testing.read().encode('utf-8')
        file_testing.close()
        file_testing = open('tests/test_hooks_files/expected_translate.txt', 'r')
        expected_data = file_testing.read().encode('utf-8')
        file_testing.close()
        process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
        output_std, output_error = process.communicate(input=input_data)
        process.kill()
        self.assertEqual(b'', output_error)
        self.assertEqual(expected_data, output_std)

    def test_update_hook(self):
        bash_command = self.settings.target_compile_update_hook
        process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
        input_data = b'one\ntwo\nthree\nfour\nfive\nsix\n\n\n\n'
        output_std, output_error = process.communicate(input=input_data)
        process.kill()
        self.assertEqual(b'', output_error)
        self.assertEqual(b'Requested files not found!\n', output_std)

        file_testing = open('tests/test_hooks_files/input_text_testing_hooks.txt', 'r')
        input_data = file_testing.read().encode('utf-8')
        file_testing.close()
        file_testing = open('tests/test_hooks_files/expected_update.txt', 'r')
        expected_data = file_testing.read().encode('utf-8')
        file_testing.close()
        process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
        output_std, output_error = process.communicate(input=input_data)
        process.kill()
        self.assertEqual(b'', output_error)
        self.assertEqual(expected_data, output_std)

    def test_finalize_hook(self):
        bash_command = self.settings.target_compile_finalize_hook
        process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
        input_data = b'one\ntwo\nthree\nfour\nfive\nsix\n\n\n\n'
        output_std, output_error = process.communicate(input=input_data)
        process.kill()
        self.assertEqual(b'', output_error)
        self.assertEqual(b'', output_std)

        file_testing = open('tests/test_hooks_files/input_text_testing_hooks.txt', 'r')
        input_data = file_testing.read().encode('utf-8')
        file_testing.close()

        process = Popen(bash_command.split(), stdout=PIPE, stderr=PIPE, stdin=PIPE)
        output_std, output_error = process.communicate(input=input_data)
        self.assertEqual(b'', output_error)
        self.assertEqual(b'', output_std)