from unittest import TestCase
from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings
from time import sleep

from navix.monitor import Monitor


class TestMonitor(TestCase):
    maxDiff = None

    def setUp(self):
        # log file path
        log_file_s = 'NaviX_start_error.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.INFO)
        self.settings = Settings()
        log = Logging()

        self.monitor = Monitor(log.queue_monitor, log.queue_logging, "TEST", self.settings)
        return

    def test_finalize(self):
        ############################################################################
        # Empty test
        ############################################################################
        fin = "NaviX.mon"
        # flush file before testing output
        open(fin, "w").close()
        data = ""
        self.monitor.finalize(data)
        line_count = 17
        sleep(2)
        with open(fin, "r") as file:
            lines = file.readlines()  # Read to end
        file.close()
        lines = lines[-line_count:]
        self.assertEqual("LastRemoteHost: \n", lines[1])
        self.assertEqual("LastRemoteIP: \n", lines[2])
        self.assertEqual("Done at: ", lines[3][:len("Done at: ")])
        self.assertEqual("JobID: \n", lines[4])
        self.assertEqual("Requested files: \n", lines[5])
        self.assertEqual("File found on server: {}\n", lines[6])
        self.assertEqual("Rank: {}\n", lines[7])
        self.assertEqual("Requirement: \n", lines[8])
        self.assertEqual("Runtime: 0\n", lines[9])
        self.assertEqual("RequestCpus: 0\n", lines[10])
        self.assertEqual("RemoteSysCpu: 0\n", lines[11])
        self.assertEqual("RemoteUserCpu: 0\n", lines[12])
        self.assertEqual("NetworkInputMb: 0\n", lines[13])
        self.assertEqual("Input_Files: []\n", lines[14])

        ############################################################################
        # Test with realistic data
        ############################################################################
        # flush file before testing output
        open(fin, "w").close()
        data = (self.settings.hpda_timestamp + " = \"15999920\"\n"
                + self.settings.last_remote_host + " = \"SuperGuenther 3000\"\n"
                + self.settings.last_remote_ip + " = \"execute-side@matchsession/123.123.123.123\"\n"
                + self.settings.network_input_mb + " = 1620\n"
                + self.settings.global_job_id + " = \"#1599920@SuperGuenther#\"\n"
                + self.settings.start_job_timestamp + " = 1599920\n"
                + self.settings.completion_timestamp + " = 1600020\n"
                + self.settings.file_found_on_server + " = \"{'/eos/opendata/cms/Run2012B/DoubleMuParked/AOD/"
                                                       + "22Jan2013-v1/10000/1EC938EF-ABEC-E211-94E0-90E6BA442F24.root'"
                                                       + ": ['[::129.13.101.145]:40009']}\"\n"
                + self.settings.remote_user_cpu + " = 1\n"
                + self.settings.remote_sys_cpu + " = 2\n"
                + self.settings.requested_cpus + " = 1\n"
                + self.settings.requested_files_tmp + " = \"root://eospublic.cern.ch//"
                                                    + "eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/"
                                                    + "10000/1EC938EF-ABEC-E211-94E0-90E6BA442F24.root\n"
                                                    + "root://eospublic.cern.ch//eos/opendata/"
                                                    + "cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/10000/"
                                                    + "4804A3F3-CDEC-E211-BC43-00259073E4EA.root\n"
                                                    + "root://eospublic.cern.ch//"
                                                    + "eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/10000/"
                                                    + "649DE9C1-EFEC-E211-B82C-485B39800C0F.root\n"
                                                    + "\"END_OF_REQUESTED_FILES\n"
                + self.settings.requested_files_mon + " = \"['/eos/opendata/cms/Run2012B/"
                                                    + "DoubleMuParked/AOD/22Jan2013-v1/"
                                                    + "10000/1EC938EF-ABEC-E211-94E0-90E6BA442F24.root','/eos/"
                                                    + "opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/10000/"
                                                    + "4804A3F3-CDEC-E211-BC43-00259073E4EA.root',"
                                                    + "'/eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/"
                                                    + "10000/649DE9C1-EFEC-E211-B82C-485B39800C0F.root']\"\n"
                + self.settings.hpda_rank_monitoring + " = \"{'machine == \"ekpsg04.ekp.kit.edu\"': "
                                                     + "{'files_found': '1'}}\"\n"
                + self.settings.hpda_requirement + " = ( machine == \"ekpsg04.ekp.kit.edu\" ) || "
                                                   + "( machine == \"ekpsg01.ekp.kit.edu\" )\n")
        self.monitor.finalize(data)
        sleep(2)
        with open(fin, "r") as file:
            lines = file.readlines()  # Read to end
        file.close()
        lines = lines[-line_count:]
        self.assertEqual("15999920: \n", lines[0])
        self.assertEqual("LastRemoteHost: SuperGuenther 3000\n", lines[1])
        self.assertEqual("LastRemoteIP: 123.123.123.123\n", lines[2])
        self.assertEqual("Done at: ", lines[3][:len("Done at: ")])
        self.assertEqual("JobID: 1599920@SuperGuenther\n", lines[4])
        self.assertEqual("Requested files: ['/eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/10000/"
                         + "1EC938EF-ABEC-E211-94E0-90E6BA442F24.root','/eos/opendata/cms/Run2012B/DoubleMuParked/"
                         + "AOD/22Jan2013-v1/10000/4804A3F3-CDEC-E211-BC43-00259073E4EA.root',"
                         + "'/eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/10000/"
                         + "649DE9C1-EFEC-E211-B82C-485B39800C0F.root']\n", lines[5])
        self.assertEqual("File found on server: {'/eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/10000/"
                         + "1EC938EF-ABEC-E211-94E0-90E6BA442F24.root': ['[::129.13.101.145]:40009']}\n", lines[6])
        self.assertEqual("Rank: {'machine == \"ekpsg04.ekp.kit.edu\"': {'files_found': '1'}}\n", lines[7])
        self.assertEqual("Requirement: ( machine == \"ekpsg04.ekp.kit.edu\" ) || "
                         + "( machine == \"ekpsg01.ekp.kit.edu\" )\n", lines[8])
        self.assertEqual("Runtime: 100.0\n", lines[9])
        self.assertEqual("RequestCpus: 1.0\n", lines[10])
        self.assertEqual("RemoteSysCpu: 2.0\n", lines[11])
        self.assertEqual("RemoteUserCpu: 1.0\n", lines[12])
        self.assertEqual("NetworkInputMb: 1620.0\n", lines[13])
        self.assertEqual("Input_Files: ['root://eospublic.cern.ch//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/"
                         + "22Jan2013-v1/10000/1EC938EF-ABEC-E211-94E0-90E6BA442F24.root:2286901718', "
                         + "'root://eospublic.cern.ch//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/"
                         + "10000/4804A3F3-CDEC-E211-BC43-00259073E4EA.root:2809716083', "
                         + "'root://eospublic.cern.ch//eos/opendata/cms/Run2012B/DoubleMuParked/AOD/22Jan2013-v1/"
                         + "10000/649DE9C1-EFEC-E211-B82C-485B39800C0F.root:2106879017']\n", lines[14])
