from unittest import TestCase
from navix.logging import *
from time import time
from tests.settings_for_test import SettingsForTesting as Settings

from navix.replygenerator.check_waiting_time import remove_listed_server_from_local_copy, set_default_weight,\
    get_servers_with_highest_rank, remove_servers_with_lower_weight, select_random_server_from_dict,\
    get_random_server_with_highest_weight, check_waiting_time_and_add_resources, known_host_to_server_dict


class TestCheckWaitingTime(TestCase):
    maxDiff = None

    def setUp(self):
        # log file path
        log_file_s = 'NaviX_start_error.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.DEBUG)
        settings = Settings()
        log = Logging()

        self.thread_num = "TEST"
        self.settings = settings
        self.logger = log.queue_logging
        return

    def test_remove_listed_server_from_local_copy(self):
        server_dict = {'machine == "test03.ekp.kit.edu"': {'count': 1, 'IP': '::178.254.445.4',
                                                           'weight': self.settings.server_weight},
                       'machine == "test02.ekp.kit.edu"': {'count': 1, 'IP': '::123.245.758.4/128',
                                                           'weight': self.settings.server_weight},
                       'machine == "test01.ekp.kit.edu"': {'count': 2, 'IP': '::1.1.1.1/120',
                                                           'weight': self.settings.server_weight}}
        local_known_servers = {'::178.254.445.4': {'name': 'machine == "test03.ekp.kit.edu"', 'weight': 50},
                               '::129.13.101.144/125': {'name': 'cloudsite == "ekpsupermachines"'},
                               '::123.245.758.4/128': {'name': 'machine == "test02.ekp.kit.edu"'},
                               '::1.1.1.1/120': {'name': 'machine == "test01.ekp.kit.edu"', 'weight': 10000},
                               '::129.13.101.148/128': {'name': 'machine == "ekpsg04.ekp.kit.edu"', 'weight': 1000},
                               '::129.13.101.149/128': {'name': 'machine == "ekpsm01.ekp.kit.edu"'}}

        local_known_servers = remove_listed_server_from_local_copy(self.logger, self.thread_num,
                                                                   server_dict, local_known_servers)
        self.assertEqual(local_known_servers, {'::129.13.101.144/125': {'name': 'cloudsite == "ekpsupermachines"'},
                                               '::129.13.101.149/128': {'name': 'machine == "ekpsm01.ekp.kit.edu"'},
                                               '::129.13.101.148/128':
                                                   {'name': 'machine == "ekpsg04.ekp.kit.edu"', 'weight': 1000}})

    def test_set_default_weight(self):
        local_known_servers = {'::178.254.445.4': {'name': 'machine == "test03.ekp.kit.edu"', 'weight': 50},
                               '::129.13.101.144/125': {'name': 'cloudsite == "ekpsupermachines"'},
                               '::123.245.758.4/128': {'name': 'machine == "test02.ekp.kit.edu"'},
                               '::1.1.1.1/120': {'name': 'machine == "test01.ekp.kit.edu"', 'weight': 10000},
                               '::129.13.101.148/128': {'name': 'machine == "ekpsg04.ekp.kit.edu"', 'weight': 1000},
                               '::129.13.101.149/128': {'name': 'machine == "ekpsm01.ekp.kit.edu"'}}
        local_known_servers = set_default_weight(self.logger, self.thread_num, local_known_servers, self.settings)
        self.assertEqual(local_known_servers, {'::178.254.445.4': {'name': 'machine == "test03.ekp.kit.edu"',
                                                                   'weight': 50},
                                               '::129.13.101.144/125': {'name': 'cloudsite == "ekpsupermachines"',
                                                                        'weight': self.settings.server_weight},
                                               '::123.245.758.4/128': {'name': 'machine == "test02.ekp.kit.edu"',
                                                                       'weight': self.settings.server_weight},
                                               '::1.1.1.1/120': {'name': 'machine == "test01.ekp.kit.edu"',
                                                                 'weight': 10000},
                                               '::129.13.101.148/128': {'name': 'machine == "ekpsg04.ekp.kit.edu"',
                                                                        'weight': 1000},
                                               '::129.13.101.149/128': {'name': 'machine == "ekpsm01.ekp.kit.edu"',
                                                                        'weight': self.settings.server_weight}})
        return

    def test_get_servers_with_highest_rank(self):
        local_known_servers = {'::178.254.445.4': {'name': 'machine == "test03.ekp.kit.edu"', 'weight': 50},
                               '::129.13.101.144/125': {'name': 'cloudsite == "ekpsupermachines"',
                                                        'weight': self.settings.server_weight},
                               '::123.245.758.4/128': {'name': 'machine == "test02.ekp.kit.edu"',
                                                       'weight': self.settings.server_weight},
                               '::1.1.1.1/120': {'name': 'machine == "test01.ekp.kit.edu"',
                                                 'weight': 10000000000},
                               '::129.13.101.148/128': {'name': 'machine == "ekpsg04.ekp.kit.edu"',
                                                        'weight': 10000000000},
                               '::129.13.101.149/128': {'name': 'machine == "ekpsm01.ekp.kit.edu"',
                                                        'weight': self.settings.server_weight}}
        self.assertEqual(get_servers_with_highest_rank(self.logger, self.thread_num, local_known_servers), 10000000000)
        return

    def test_remove_servers_with_lower_weight(self):
        local_known_servers = {'::178.254.445.4': {'name': 'machine == "test03.ekp.kit.edu"', 'weight': 50},
                               '::129.13.101.144/125': {'name': 'cloudsite == "ekpsupermachines"',
                                                        'weight': self.settings.server_weight},
                               '::123.245.758.4/128': {'name': 'machine == "test02.ekp.kit.edu"',
                                                       'weight': self.settings.server_weight},
                               '::1.1.1.1/120': {'name': 'machine == "test01.ekp.kit.edu"', 'weight': 10000000000},
                               '::129.13.101.148/128': {'name': 'machine == "ekpsg04.ekp.kit.edu"',
                                                        'weight': 10000000000},
                               '::129.13.101.149/128': {'name': 'machine == "ekpsm01.ekp.kit.edu"',
                                                        'weight': self.settings.server_weight}}
        weight = 10000000
        local_known_servers = remove_servers_with_lower_weight(self.logger, self.thread_num,
                                                               local_known_servers, weight)
        self.assertEqual(local_known_servers, {'::1.1.1.1/120': {'name': 'machine == "test01.ekp.kit.edu"',
                                                                 'weight': 10000000000},
                                               '::129.13.101.148/128': {'name': 'machine == "ekpsg04.ekp.kit.edu"',
                                                                        'weight': 10000000000}})
        return

    def test_select_random_server_from_dict(self):
        server_dict = dict()
        local_known_servers = {'::178.254.445.4': {'name': 'machine == "test03.ekp.kit.edu"', 'weight': 50},
                               '::129.13.101.144/125': {'name': 'cloudsite == "ekpsupermachines"',
                                                        'weight': self.settings.server_weight},
                               '::123.245.758.4/128': {'name': 'machine == "test02.ekp.kit.edu"',
                                                       'weight': self.settings.server_weight},
                               '::1.1.1.1/120': {'name': 'machine == "test01.ekp.kit.edu"', 'weight': 10000000000},
                               '::129.13.101.148/128': {'name': 'machine == "ekpsg04.ekp.kit.edu"',
                                                        'weight': 10000000000},
                               '::129.13.101.149/128': {'name': 'machine == "ekpsm01.ekp.kit.edu"',
                                                        'weight': self.settings.server_weight}}
        server_dict = select_random_server_from_dict(self.logger, self.thread_num, local_known_servers,
                                                     server_dict, self.settings)
        only_one = False
        for server, value in iter(server_dict.items()):
            if local_known_servers[value.get('IP')] is None:
                only_one = False
                break
            elif only_one is False:
                only_one = True
            else:
                only_one = False
                break
        self.assertTrue(only_one)
        return

    def test_get_random_server_with_highest_weight(self):
        server_dict = dict()
        server_dict = get_random_server_with_highest_weight(self.logger, self.thread_num, self.settings, server_dict)
        try:
            self.assertEqual(server_dict, {'machine == "ekpsg02.ekp.kit.edu"': {'IP': '::810d:6592/128',
                                                                                'weight': 10000, 'count': 1}})

            self.logger.put("DEBUG: Tested function: 'get_random_server_with_highest_weight' appears to be fine.")
        except AssertionError:
            self.logger.put("DEBUG: " + "Tested function: 'get_random_server_with_highest_weight': Test failed. \n \
                                    This tests depends on the server list read from the config! \n \
                                    You may have to modify this tests to match! \n \
                                    If this will not be fixed with modifying the code - deeper \
                                    code inspection is suggested")
        return

    def test_check_waiting_time_and_add_resources(self):
        last_time = time()
        server_dict = dict()
        check_waiting_time_and_add_resources(self.logger, self.thread_num, server_dict, last_time, self.settings)
        self.assertEqual(server_dict, {})
        last_time = time() - (self.settings.time_before_adding_additional_resources + 1)
        check_waiting_time_and_add_resources(self.logger, self.thread_num, server_dict, last_time, self.settings)
        self.assertNotEqual(server_dict, {})
        return

    def test_known_host_to_server(self):
        server_dict = dict()
        server = dict()
        server['name'] = 'cloudsite == "IPV6-Test"'
        ip = '::129.13.101.144'
        server_dict = known_host_to_server_dict(server, ip, server_dict, self.settings)
        self.assertEqual(server_dict,
                         {'cloudsite == "IPV6-Test"': {'weight': self.settings.server_weight,
                                                       'IP': '::129.13.101.144', 'count': 1}})
        return

    def test_get_random_server_with_highest_weight_corrupted_dict(self):
        server_dict = [] # no dict!
        with self.assertRaises(ValueError) as err:
            get_random_server_with_highest_weight(self.logger, self.thread_num, self.settings, server_dict)
        self.assertEqual("'list' object has no attribute 'items'", str(err.exception))

