from unittest import TestCase
from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings
from time import time

from navix.replygenerator.create_reply import create_hpda_rank, add_hpda_requirement, add_timestamp, \
    monitoring_rank, add_classad_for_monitoring_to_reply


class TestCreateReply(TestCase):
    maxDiff = None

    def setUp(self):
        # log file path
        log_file_s = 'NaviX_start_error.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.INFO)
        settings = Settings()
        log = Logging()

        self.thread_num = "TEST"
        self.settings = settings
        self.logger = log.queue_logging

        return

    def test_create_location_string(self):
        count_found_files = 0
        server_dict = dict()
        hpda_rank, monitor_rank_server = create_hpda_rank(self.logger, self.thread_num, server_dict, count_found_files,
                                                          self.settings)
        self.assertEqual(hpda_rank, "HPDA_RANK = (0)\n")
        count_found_files = 1
        server_dict = {'machine == "test03.ekp.kit.edu"': {'count': 1, 'IP': '::178.254.445.4',
                                                           'weight': self.settings.server_weight},
                       'machine == "test02.ekp.kit.edu"': {'count': 1, 'IP': '::123.245.758.4/128',
                                                           'weight': self.settings.server_weight}}

        hpda_rank, monitor_rank_server = create_hpda_rank(self.logger, self.thread_num, server_dict, count_found_files,
                                                          self.settings)
        self.assertTrue(hpda_rank == 'HPDA_RANK = (machine == "test03.ekp.kit.edu") *1000.0 '
                        + '+ (machine == "test02.ekp.kit.edu") *1000.0'
                        or 'HPDA_RANK = (machine == "test02.ekp.kit.edu") *1000.0'
                        + ' + (machine == "test03.ekp.kit.edu") *1000.0')
        return

    def test_add_hpda_requirement(self):
        count_found_files = 0
        server_dict = dict()
        last_time = time()

        requirement_string = add_hpda_requirement(self.logger, self.thread_num, count_found_files, server_dict,
                                                  last_time, self.settings)
        self.assertEqual(requirement_string, "HPDA_REQUIREMENT = (True)\n")

        count_found_files = 1
        server_dict = {'machine == "test03.ekp.kit.edu"': {'count': 1, 'IP': '::178.254.445.4',
                                                           'weight': self.settings.server_weight},
                       'machine == "test02.ekp.kit.edu"': {'count': 1, 'IP': '::123.245.758.4/128',
                                                           'weight': self.settings.server_weight}}
        requirement_string = add_hpda_requirement(self.logger, self.thread_num, count_found_files, server_dict,
                                                  last_time, self.settings)
        self.assertTrue(requirement_string == 'HPDA_REQUIREMENT = ( machine == "test03.ekp.kit.edu" ) || '
                        + '( machine == "test02.ekp.kit.edu" )\n'
                        or 'HPDA_REQUIREMENT = ( machine == "test02.ekp.kit.edu )" || '
                        + '( machine == "test03.ekp.kit.edu" )\n')
        return

    def test_add_timestamp(self):
        last_time = time()
        reply = ""
        reply = add_timestamp(self.logger, self.thread_num, reply, last_time, self.settings)
        self.assertFalse(reply)
        reply = ""
        last_time = None
        reply = add_timestamp(self.logger, self.thread_num, reply, last_time, self.settings)
        self.assertTrue(reply.__contains__("HPDA_TIMESTAMP = \""))
        return

    def test_monitoring_rank(self):
        server = 'test'
        count = 5
        ip = '192.168.178.2'
        count_found_files = 15
        weight = 10
        monitor_rank_server = monitoring_rank(server, count, ip, count_found_files, weight)
        self.assertEqual(monitor_rank_server['test']['files_found'], 15)
        self.assertEqual(monitor_rank_server['test']['count'], 5)
        self.assertEqual(monitor_rank_server['test']['IP'], '192.168.178.2')
        self.assertEqual(monitor_rank_server['test']['weight'], 10)
        return

    def test_add_classad_for_monitoring_to_reply(self):
        reply = "this is some \nrandom text\n"
        files_server_dict = {'::178.254.445.4': {'name': 'machine == "test03.ekp.kit.edu"'}}
        files_to_search_array = ['blaaaahhh', 'blub', 'blibbb']
        monitor_rank_server = {'machine == "test01.ekp.kit.edu"': {'IP': '::1.1.1.1/120'}}
        reply = add_classad_for_monitoring_to_reply(self.settings, reply, files_server_dict, files_to_search_array,
                                                    monitor_rank_server)
        self.assertEqual(reply, 'this is some '
                         + '\nrandom text'
                         + "\nFile_found_on_server = \"{'::178.254.445.4': "
                         + "{'name': 'machine == \"test03.ekp.kit.edu\"'}}\""
                         + "\nRequested_files = \"['blaaaahhh', 'blub', 'blibbb']\""
                         + "\nHPDA_RANK_MONITORING = \"{'machine == \\\"test01.ekp.kit.edu\\\"': "
                         + "{'IP': '::1.1.1.1/120'}}\"\n")
        return
