from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings
from unittest import TestCase

from navix.replygenerator.generate_file_list import get_files_to_search_from_data, create_file_list_to_search


class TestGenerateFileList(TestCase):
    maxDiff = None

    def setUp(self):
        # log file path
        log_file_s = 'NaviX_start_error.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.DEBUG)
        settings = Settings()
        log = Logging()

        self.thread_num = "TEST"
        self.settings = settings
        self.logger = log.queue_logging
        return

    def test_get_files_to_search_from_data(self):
        # test various combinations of one and three files listed
        data = "root://server.blob//address"
        expected_input_tmp = data
        files_to_search_array, input_tmp = get_files_to_search_from_data(self.logger, self.thread_num, data)
        self.assertEqual(files_to_search_array, ['/address'])
        self.assertEqual(input_tmp, expected_input_tmp)

        data = "root://server.blob//address1,root://server.blob//address2,root://server.blob//address3"
        expected_input_tmp = data
        files_to_search_array, input_tmp = get_files_to_search_from_data(self.logger, self.thread_num, data)
        self.assertEqual(files_to_search_array, ['/address1', '/address2', '/address3'])
        self.assertEqual(input_tmp, expected_input_tmp)

        data = "root://server.blob//address1, root://server.blob//address2, root://server.blob//address3"
        files_to_search_array, input_tmp = get_files_to_search_from_data(self.logger, self.thread_num, data)
        self.assertEqual(files_to_search_array, ['/address1', '/address2', '/address3'])
        self.assertEqual(input_tmp, expected_input_tmp)

        data = "root://server.blob//address1,\n root://server.blob//address2,\n root://server.blob//address3"
        files_to_search_array, input_tmp = get_files_to_search_from_data(self.logger, self.thread_num, data)
        self.assertEqual(files_to_search_array, ['/address1', '/address2', '/address3'])
        self.assertEqual(input_tmp, expected_input_tmp)

        data = "root://server.blob//address1, \n root://server.blob//address2, \n root://server.blob//address3"
        files_to_search_array, input_tmp = get_files_to_search_from_data(self.logger, self.thread_num, data)
        self.assertEqual(files_to_search_array, ['/address1', '/address2', '/address3'])
        self.assertEqual(input_tmp, expected_input_tmp)

        data = "root://server.blob//address1 \n root://server.blob//address2\n root://server.blob//address3"
        files_to_search_array, input_tmp = get_files_to_search_from_data(self.logger, self.thread_num, data)
        self.assertEqual(files_to_search_array, ['/address1', '/address2', '/address3'])
        self.assertEqual(input_tmp, expected_input_tmp)

        data = "root:server.blob//address1 \n root://server.blob//address2\n root://server.blob//address3"
        expected_input_tmp = "root:server.blob//address1,root://server.blob//address2,root://server.blob//address3"
        files_to_search_array, input_tmp = get_files_to_search_from_data(self.logger, self.thread_num, data)
        self.assertEqual(files_to_search_array, ['/address2', '/address3'])
        self.assertEqual(input_tmp, expected_input_tmp)

        data = "root://address1 \n root://server.blob//address2\n root://server.blob//address3"
        expected_input_tmp = "root://address1,root://server.blob//address2,root://server.blob//address3"
        files_to_search_array, input_tmp = get_files_to_search_from_data(self.logger, self.thread_num, data)
        self.assertEqual(files_to_search_array, ['/address2', '/address3'])
        self.assertEqual(input_tmp, expected_input_tmp)
        
        data = "file://PreRAWskimmed_53632.root"
        expected_input_tmp = "file://PreRAWskimmed_53632.root"
        files_to_search_array, input_tmp = get_files_to_search_from_data(self.logger, self.thread_num, data)
        print(files_to_search_array, input_tmp)
        self.assertEqual(files_to_search_array, [])
        self.assertEqual(input_tmp, expected_input_tmp)
        
        self.logger.put("DEBUG: Tested function: 'test_get_files_to_search_from_data' appears to be fine.")
        return

    def test_create_file_list_to_search(self):
        data = ('Input_Files = "root://server.blob//file1,root://server.blob//file2,'
                + 'root://server.blob//file3"')
        expected_input_tmp = 'root://server.blob//file1,root://server.blob//file2,root://server.blob//file3'
        files_to_search_array, input_tmp = create_file_list_to_search(self.logger, self.thread_num, data)
        self.assertEqual(files_to_search_array, ['/file1', '/file2', '/file3'])
        self.assertEqual(input_tmp, expected_input_tmp)
        self.logger.put("DEBUG: Tested function: 'create_file_list_to_search' appears to be fine.")
        return
