from unittest import TestCase
from navix.logging import *
from tests.settings_for_test import SettingsForTesting as Settings

from navix.replygenerator.locate_files import known_host_to_server_dict, find_server_to_location, get_location_of_files,\
    search_for_files_pyxrootd, add_file_to_dict_for_monitoring_classad


class TestLocateFiles(TestCase):
    maxDiff = None

    def setUp(self):
        # log file path
        log_file_s = 'NaviX_start_error.log'

        # create logger
        start_log = logging.getLogger(log_file_s)
        start_log.setLevel(logging.DEBUG)
        settings = Settings()
        log = Logging()

        self.thread_num = "TEST"
        self.settings = settings
        self.logger = log.queue_logging

        self.server_dict = dict()
        return

    def test_known_host_to_server_dict(self):
        # self, server, IP'
        ip = "::1.1.1.1/120"
        server = {'name': 'machine == "test01.ekp.kit.edu"'}
        server_dict = known_host_to_server_dict(server, ip, self.settings, self.server_dict)

        self.assertEqual(server_dict, {'machine == "test01.ekp.kit.edu"':
                                       {'count': 1, 'IP': '::1.1.1.1/120',
                                        'weight': self.settings.server_weight}})

        server_dict = known_host_to_server_dict(server, ip, self.settings, self.server_dict)
        self.assertEqual(server_dict, {'machine == "test01.ekp.kit.edu"':
                                       {'count': 2, 'IP': '::1.1.1.1/120',
                                        'weight': self.settings.server_weight}})
        ip = "::123.245.758.4/128"
        server = {'name': 'machine == "test02.ekp.kit.edu"'}
        server_dict = known_host_to_server_dict(server, ip, self.settings, self.server_dict)
        self.assertEqual(server_dict, {'machine == "test02.ekp.kit.edu"':
                                       {'count': 1, 'IP': '::123.245.758.4/128',
                                        'weight': self.settings.server_weight},
                                       'machine == "test01.ekp.kit.edu"':
                                       {'count': 2, 'IP': '::1.1.1.1/120',
                                        'weight': self.settings.server_weight}})
        ip = "::178.254.445.4"
        server = {'name': 'machine == "test03.ekp.kit.edu"'}
        server_dict = known_host_to_server_dict(server, ip, self.settings, self.server_dict)
        self.assertEqual(server_dict, {'machine == "test03.ekp.kit.edu"':
                                       {'count': 1, 'IP': '::178.254.445.4',
                                        'weight': self.settings.server_weight},
                                       'machine == "test02.ekp.kit.edu"':
                                       {'count': 1, 'IP': '::123.245.758.4/128',
                                        'weight': self.settings.server_weight},
                                       'machine == "test01.ekp.kit.edu"':
                                       {'count': 2, 'IP': '::1.1.1.1/120',
                                        'weight': self.settings.server_weight}})
        return

    def test_find_server_to_location(self):
        server_dict = dict()
        server_dict = find_server_to_location(self.logger, self.thread_num, "[::1.3.2.5]:358631",
                                              self.settings, server_dict)
        self.assertEqual(server_dict, {})
        server_dict = find_server_to_location(self.logger, self.thread_num, "[::129.13.101.149]:358631",
                                              self.settings, server_dict)
        self.assertTrue(server_dict == {'cloudsite == "ekpsupermachines"':
                                        {'count': 1, 'IP': '::810d:6590/125',
                                         'weight': self.settings.server_weight},
                                        'machine == "ekpsm01.ekp.kit.edu"':
                                        {'count': 1, 'IP': '::810d:6595/128',
                                         'weight': self.settings.server_weight}} or
                                       {'machine == "ekpsm01.ekp.kit.edu"':
                                        {'count': 1, 'IP': '::810d:6595/128',
                                         'weight': self.settings.server_weight},
                                        'cloudsite == "ekpsupermachines"':
                                        {'count': 1, 'IP': '::810d:6590/125',
                                         'weight': self.settings.server_weight}})
        return

    def test_add_file_to_dict_for_monitoring_classad(self):
        file_name = "test"
        server_ip = '192.168.178.1'
        files_server_dict = dict()
        add_file_to_dict_for_monitoring_classad(file_name, server_ip, files_server_dict)
        self.assertEqual(files_server_dict, {'test': ['192.168.178.1']})
        file_name = 'test2'
        server_ip = '192.168.178.1'
        add_file_to_dict_for_monitoring_classad(file_name, server_ip, files_server_dict)
        self.assertTrue(files_server_dict == {'test': ['192.168.178.1'], 'test2': ['192.168.178.1']}
                        or {'test2': ['192.168.178.1'], 'test': ['192.168.178.1']})
        server_ip = '192.168.178.2'
        add_file_to_dict_for_monitoring_classad(file_name, server_ip, files_server_dict)
        self.assertTrue(files_server_dict == {'test': ['192.168.178.1'], 'test2': ['192.168.178.1', '192.168.178.2']}
                        or {'test2': ['192.168.178.1', '192.168.178.2'], 'test': ['192.168.178.1']})
        return

    def test_get_location_of_files(self):
        # TODO request pyxrootd and xrootd server!
        return
        get_location_of_files

    def test_search_for_files_pyxrootd(self):
        # TODO request pyxrootd and xrootd server!
        return
        search_for_files_pyxrootd
